import json as json_
import os
import warnings
from pathlib import Path
from typing import Any, override

from liblaf import grapes

from . import AbstractSerializer


class JSONSerializer(AbstractSerializer):
    @override
    def load(self, fpath: str | os.PathLike[str], **kwargs) -> Any:
        fpath: Path = grapes.as_path(fpath)
        with fpath.open() as fp:
            return json_.load(fp, **kwargs)

    @override
    def loads(self, data: str, **kwargs) -> Any:
        return json_.loads(data, **kwargs)

    @override
    def dump(self, fpath: str | os.PathLike[str], data: Any, **kwargs) -> None:
        fpath: Path = grapes.as_path(fpath)
        with fpath.open("w") as fp:
            json_.dump(data, fp, **kwargs)

    @override
    def dumps(self, data: Any, **kwargs) -> str:
        return json_.dumps(data, **kwargs)


json = JSONSerializer()
load_json = json.load
loads_json = json.loads
dump_json = json.dump
dumps_json = json.dumps


@warnings.deprecated("Use `dump_json()` instead of `save_json()`")
def save_json(fpath: str | os.PathLike[str], data: Any) -> None:
    """Save data to a JSON file.

    Args:
        fpath: The file path where the JSON data will be saved.
        data: The data to be serialized to JSON and saved to the file.
    """
    fpath: Path = Path(fpath)
    with fpath.open("w") as fp:
        json.dump(data, fp)



from ._abc import AbstractSerializer
from ._json import (
    JSONSerializer,
    dump_json,
    dumps_json,
    json,
    load_json,
    loads_json,
    save_json,
)
from ._pydantic import load_pydantic, save_pydantic
from ._serde import READERS, WRITERS, deserialize, serialize
from ._toml import TOMLSerializer, dump_toml, dumps_toml, load_toml, save_toml, toml
from ._yaml import (
    YAMLSerializer,
    dump_yaml,
    dumps_yaml,
    load_yaml,
    loads_yaml,
    save_yaml,
    yaml,
)

__all__ = [
    "READERS",
    "WRITERS",
    "AbstractSerializer",
    "JSONSerializer",
    "TOMLSerializer",
    "YAMLSerializer",
    "deserialize",
    "dump_json",
    "dump_toml",
    "dump_yaml",
    "dumps_json",
    "dumps_toml",
    "dumps_yaml",
    "json",
    "load_json",
    "load_pydantic",
    "load_toml",
    "load_yaml",
    "loads_json",
    "loads_yaml",
    "save_json",
    "save_pydantic",
    "save_toml",
    "save_yaml",
    "serialize",
    "toml",
    "yaml",
]



import io
import os
import warnings
from pathlib import Path
from typing import Any

from liblaf import grapes

from . import AbstractSerializer

with grapes.optional_imports(extra="serde"):
    from ruamel.yaml import YAML


class YAMLSerializer(AbstractSerializer):
    def load(self, fpath: str | os.PathLike[str], **kwargs) -> Any:
        fpath: Path = grapes.as_path(fpath)
        yaml = YAML(**kwargs)
        with fpath.open() as fp:
            return yaml.load(fp)

    def loads(self, data: str, **kwargs) -> Any:
        stream = io.StringIO(data)
        yaml = YAML(**kwargs)
        return yaml.load(stream)

    def dump(self, fpath: str | os.PathLike[str], data: Any, **kwargs) -> None:
        fpath: Path = grapes.as_path(fpath)
        yaml = YAML(**kwargs)
        with fpath.open("w") as fp:
            yaml.dump(data, fp)

    def dumps(self, data: Any, **kwargs) -> str:
        stream = io.StringIO()
        yaml = YAML(**kwargs)
        yaml.dump(data, stream)
        return stream.getvalue()


yaml = YAMLSerializer()
load_yaml = yaml.load
loads_yaml = yaml.loads
dump_yaml = yaml.dump
dumps_yaml = yaml.dumps


@warnings.deprecated("Use `dump_yaml()` instead of `save_yaml()`")
def save_yaml(fpath: str | os.PathLike[str], data: Any) -> None:
    fpath: Path = Path(fpath)
    yaml = YAML()
    with fpath.open("w") as fp:
        yaml.dump(data, fp)



import os
import warnings
from pathlib import Path
from typing import Any, override

from liblaf import grapes

from . import AbstractSerializer

with grapes.optional_imports(extra="toml"):
    import tomlkit


class TOMLSerializer(AbstractSerializer):
    @override
    def load(self, fpath: str | os.PathLike[str], **kwargs) -> tomlkit.TOMLDocument:
        fpath: Path = grapes.as_path(fpath)
        with fpath.open() as fp:
            return tomlkit.load(fp, **kwargs)

    @override
    def loads(self, data: str, **kwargs) -> tomlkit.TOMLDocument:
        return tomlkit.loads(data, **kwargs)

    @override
    def dump(self, fpath: str | os.PathLike[str], data: Any, **kwargs) -> None:
        fpath: Path = grapes.as_path(fpath)
        with fpath.open("w") as fp:
            tomlkit.dump(data, fp, **kwargs)

    @override
    def dumps(self, data: Any, **kwargs) -> str:
        return tomlkit.dumps(data, **kwargs)


toml = TOMLSerializer()
load_toml = toml.load
loads_toml = toml.loads
dump_toml = toml.dump
dumps_toml = toml.dumps


@warnings.deprecated("Use `dump_toml()` instead of `save_toml()`")
def save_toml(
    fpath: str | os.PathLike[str], data: Any, *, sort_keys: bool = False
) -> None:
    """Save data to a TOML file.

    Args:
        fpath: The file path where the TOML data will be saved.
        data: The data to be serialized and saved in TOML format.
        sort_keys: Whether to sort the keys in the output.
    """
    fpath: Path = Path(fpath)
    with fpath.open("w") as fp:
        tomlkit.dump(data, fp, sort_keys=sort_keys)
